const BaseSelect = require('pim/form/common/fields/select');

class TimezoneField extends BaseSelect {
  /**
   * {@inheritdoc}
   */
  configure() {
    BaseSelect.prototype.configure.apply(this, arguments);
    this.config.choices = {
      'Africa': {
        'Africa/Abidjan': 'Abidjan GMT (UTC+00:00)',
        'Africa/Accra': 'Accra GMT (UTC+00:00)',
        'Africa/Addis_Ababa': 'Addis Ababa EAT (UTC+03:00)',
        'Africa/Algiers': 'Algiers CET (UTC+01:00)',
        'Africa/Asmara': 'Asmara EAT (UTC+03:00)',
        'Africa/Bamako': 'Bamako GMT (UTC+00:00)',
        'Africa/Bangui': 'Bangui WAT (UTC+01:00)',
        'Africa/Banjul': 'Banjul GMT (UTC+00:00)',
        'Africa/Bissau': 'Bissau GMT (UTC+00:00)',
        'Africa/Blantyre': 'Blantyre CAT (UTC+02:00)',
        'Africa/Brazzaville': 'Brazzaville WAT (UTC+01:00)',
        'Africa/Bujumbura': 'Bujumbura CAT (UTC+02:00)',
        'Africa/Cairo': 'Cairo EET (UTC+02:00)',
        'Africa/Casablanca': 'Casablanca WEST (UTC+01:00)',
        'Africa/Ceuta': 'Ceuta CEST (UTC+02:00)',
        'Africa/Conakry': 'Conakry GMT (UTC+00:00)',
        'Africa/Dakar': 'Dakar GMT (UTC+00:00)',
        'Africa/Dar_es_Salaam': 'Dar es Salaam EAT (UTC+03:00)',
        'Africa/Djibouti': 'Djibouti EAT (UTC+03:00)',
        'Africa/Douala': 'Douala WAT (UTC+01:00)',
        'Africa/El_Aaiun': 'El Aaiun WEST (UTC+01:00)',
        'Africa/Freetown': 'Freetown GMT (UTC+00:00)',
        'Africa/Gaborone': 'Gaborone CAT (UTC+02:00)',
        'Africa/Harare': 'Harare CAT (UTC+02:00)',
        'Africa/Johannesburg': 'Johannesburg SAST (UTC+02:00)',
        'Africa/Juba': 'Juba EAT (UTC+03:00)',
        'Africa/Kampala': 'Kampala EAT (UTC+03:00)',
        'Africa/Khartoum': 'Khartoum EAT (UTC+03:00)',
        'Africa/Kigali': 'Kigali CAT (UTC+02:00)',
        'Africa/Kinshasa': 'Kinshasa WAT (UTC+01:00)',
        'Africa/Lagos': 'Lagos WAT (UTC+01:00)',
        'Africa/Libreville': 'Libreville WAT (UTC+01:00)',
        'Africa/Lome': 'Lome GMT (UTC+00:00)',
        'Africa/Luanda': 'Luanda WAT (UTC+01:00)',
        'Africa/Lubumbashi': 'Lubumbashi CAT (UTC+02:00)',
        'Africa/Lusaka': 'Lusaka CAT (UTC+02:00)',
        'Africa/Malabo': 'Malabo WAT (UTC+01:00)',
        'Africa/Maputo': 'Maputo CAT (UTC+02:00)',
        'Africa/Maseru': 'Maseru SAST (UTC+02:00)',
        'Africa/Mbabane': 'Mbabane SAST (UTC+02:00)',
        'Africa/Mogadishu': 'Mogadishu EAT (UTC+03:00)',
        'Africa/Monrovia': 'Monrovia GMT (UTC+00:00)',
        'Africa/Nairobi': 'Nairobi EAT (UTC+03:00)',
        'Africa/Ndjamena': 'Ndjamena WAT (UTC+01:00)',
        'Africa/Niamey': 'Niamey WAT (UTC+01:00)',
        'Africa/Nouakchott': 'Nouakchott GMT (UTC+00:00)',
        'Africa/Ouagadougou': 'Ouagadougou GMT (UTC+00:00)',
        'Africa/Porto-Novo': 'Porto-Novo WAT (UTC+01:00)',
        'Africa/Sao_Tome': 'Sao Tome GMT (UTC+00:00)',
        'Africa/Tripoli': 'Tripoli EET (UTC+02:00)',
        'Africa/Tunis': 'Tunis CET (UTC+01:00)',
        'Africa/Windhoek': 'Windhoek WAT (UTC+01:00)'
      },
      'America': {
        'America/Adak': 'Adak HDT (UTC-09:00)',
        'America/Anchorage': 'Anchorage AKDT (UTC-08:00)',
        'America/Anguilla': 'Anguilla AST (UTC-04:00)',
        'America/Antigua': 'Antigua AST (UTC-04:00)',
        'America/Araguaina': 'Araguaina BRT (UTC-03:00)',
        'America/Argentina/Buenos_Aires': 'Argentina - Buenos Aires ART (UTC-03:00)',
        'America/Argentina/Catamarca': 'Argentina - Catamarca ART (UTC-03:00)',
        'America/Argentina/Cordoba': 'Argentina - Cordoba ART (UTC-03:00)',
        'America/Argentina/Jujuy': 'Argentina - Jujuy ART (UTC-03:00)',
        'America/Argentina/La_Rioja': 'Argentina - La Rioja ART (UTC-03:00)',
        'America/Argentina/Mendoza': 'Argentina - Mendoza ART (UTC-03:00)',
        'America/Argentina/Rio_Gallegos': 'Argentina - Rio Gallegos ART (UTC-03:00)',
        'America/Argentina/Salta': 'Argentina - Salta ART (UTC-03:00)',
        'America/Argentina/San_Juan': 'Argentina - San Juan ART (UTC-03:00)',
        'America/Argentina/San_Luis': 'Argentina - San Luis ART (UTC-03:00)',
        'America/Argentina/Tucuman': 'Argentina - Tucuman ART (UTC-03:00)',
        'America/Argentina/Ushuaia': 'Argentina - Ushuaia ART (UTC-03:00)',
        'America/Aruba': 'Aruba AST (UTC-04:00)',
        'America/Asuncion': 'Asuncion PYT (UTC-04:00)',
        'America/Atikokan': 'Atikokan EST (UTC-05:00)',
        'America/Bahia': 'Bahia BRT (UTC-03:00)',
        'America/Bahia_Banderas': 'Bahia Banderas CDT (UTC-05:00)',
        'America/Barbados': 'Barbados AST (UTC-04:00)',
        'America/Belem': 'Belem BRT (UTC-03:00)',
        'America/Belize': 'Belize CST (UTC-06:00)',
        'America/Blanc-Sablon': 'Blanc-Sablon AST (UTC-04:00)',
        'America/Boa_Vista': 'Boa Vista AMT (UTC-04:00)',
        'America/Bogota': 'Bogota COT (UTC-05:00)',
        'America/Boise': 'Boise MDT (UTC-06:00)',
        'America/Cambridge_Bay': 'Cambridge Bay MDT (UTC-06:00)',
        'America/Campo_Grande': 'Campo Grande AMT (UTC-04:00)',
        'America/Cancun': 'Cancun EST (UTC-05:00)',
        'America/Caracas': 'Caracas VET (UTC-04:00)',
        'America/Cayenne': 'Cayenne GFT (UTC-03:00)',
        'America/Cayman': 'Cayman EST (UTC-05:00)',
        'America/Chicago': 'Chicago CDT (UTC-05:00)',
        'America/Chihuahua': 'Chihuahua MDT (UTC-06:00)',
        'America/Costa_Rica': 'Costa Rica CST (UTC-06:00)',
        'America/Creston': 'Creston MST (UTC-07:00)',
        'America/Cuiaba': 'Cuiaba AMT (UTC-04:00)',
        'America/Curacao': 'Curacao AST (UTC-04:00)',
        'America/Danmarkshavn': 'Danmarkshavn GMT (UTC+00:00)',
        'America/Dawson': 'Dawson PDT (UTC-07:00)',
        'America/Dawson_Creek': 'Dawson Creek MST (UTC-07:00)',
        'America/Denver': 'Denver MDT (UTC-06:00)',
        'America/Detroit': 'Detroit EDT (UTC-04:00)',
        'America/Dominica': 'Dominica AST (UTC-04:00)',
        'America/Edmonton': 'Edmonton MDT (UTC-06:00)',
        'America/Eirunepe': 'Eirunepe ACT (UTC-05:00)',
        'America/El_Salvador': 'El Salvador CST (UTC-06:00)',
        'America/Fort_Nelson': 'Fort Nelson MST (UTC-07:00)',
        'America/Fortaleza': 'Fortaleza BRT (UTC-03:00)',
        'America/Glace_Bay': 'Glace Bay ADT (UTC-03:00)',
        'America/Godthab': 'Godthab WGST (UTC-02:00)',
        'America/Goose_Bay': 'Goose Bay ADT (UTC-03:00)',
        'America/Grand_Turk': 'Grand Turk AST (UTC-04:00)',
        'America/Grenada': 'Grenada AST (UTC-04:00)',
        'America/Guadeloupe': 'Guadeloupe AST (UTC-04:00)',
        'America/Guatemala': 'Guatemala CST (UTC-06:00)',
        'America/Guayaquil': 'Guayaquil ECT (UTC-05:00)',
        'America/Guyana': 'Guyana GYT (UTC-04:00)',
        'America/Halifax': 'Halifax ADT (UTC-03:00)',
        'America/Havana': 'Havana CDT (UTC-04:00)',
        'America/Hermosillo': 'Hermosillo MST (UTC-07:00)',
        'America/Indiana/Indianapolis': 'Indiana - Indianapolis EDT (UTC-04:00)',
        'America/Indiana/Knox': 'Indiana - Knox CDT (UTC-05:00)',
        'America/Indiana/Marengo': 'Indiana - Marengo EDT (UTC-04:00)',
        'America/Indiana/Petersburg': 'Indiana - Petersburg EDT (UTC-04:00)',
        'America/Indiana/Tell_City': 'Indiana - Tell City CDT (UTC-05:00)',
        'America/Indiana/Vevay': 'Indiana - Vevay EDT (UTC-04:00)',
        'America/Indiana/Vincennes': 'Indiana - Vincennes EDT (UTC-04:00)',
        'America/Indiana/Winamac': 'Indiana - Winamac EDT (UTC-04:00)',
        'America/Inuvik': 'Inuvik MDT (UTC-06:00)',
        'America/Iqaluit': 'Iqaluit EDT (UTC-04:00)',
        'America/Jamaica': 'Jamaica EST (UTC-05:00)',
        'America/Juneau': 'Juneau AKDT (UTC-08:00)',
        'America/Kentucky/Louisville': 'Kentucky - Louisville EDT (UTC-04:00)',
        'America/Kentucky/Monticello': 'Kentucky - Monticello EDT (UTC-04:00)',
        'America/Kralendijk': 'Kralendijk AST (UTC-04:00)',
        'America/La_Paz': 'La Paz BOT (UTC-04:00)',
        'America/Lima': 'Lima PET (UTC-05:00)',
        'America/Los_Angeles': 'Los Angeles PDT (UTC-07:00)',
        'America/Lower_Princes': 'Lower Princes AST (UTC-04:00)',
        'America/Maceio': 'Maceio BRT (UTC-03:00)',
        'America/Managua': 'Managua CST (UTC-06:00)',
        'America/Manaus': 'Manaus AMT (UTC-04:00)',
        'America/Marigot': 'Marigot AST (UTC-04:00)',
        'America/Martinique': 'Martinique AST (UTC-04:00)',
        'America/Matamoros': 'Matamoros CDT (UTC-05:00)',
        'America/Mazatlan': 'Mazatlan MDT (UTC-06:00)',
        'America/Menominee': 'Menominee CDT (UTC-05:00)',
        'America/Merida': 'Merida CDT (UTC-05:00)',
        'America/Metlakatla': 'Metlakatla AKDT (UTC-08:00)',
        'America/Mexico_City': 'Mexico City CDT (UTC-05:00)',
        'America/Miquelon': 'Miquelon PMDT (UTC-02:00)',
        'America/Moncton': 'Moncton ADT (UTC-03:00)',
        'America/Monterrey': 'Monterrey CDT (UTC-05:00)',
        'America/Montevideo': 'Montevideo UYT (UTC-03:00)',
        'America/Montserrat': 'Montserrat AST (UTC-04:00)',
        'America/Nassau': 'Nassau EDT (UTC-04:00)',
        'America/New_York': 'New York EDT (UTC-04:00)',
        'America/Nipigon': 'Nipigon EDT (UTC-04:00)',
        'America/Nome': 'Nome AKDT (UTC-08:00)',
        'America/Noronha': 'Noronha FNT (UTC-02:00)',
        'America/North_Dakota/Beulah': 'North Dakota - Beulah CDT (UTC-05:00)',
        'America/North_Dakota/Center': 'North Dakota - Center CDT (UTC-05:00)',
        'America/North_Dakota/New_Salem': 'North Dakota - New Salem CDT (UTC-05:00)',
        'America/Ojinaga': 'Ojinaga MDT (UTC-06:00)',
        'America/Panama': 'Panama EST (UTC-05:00)',
        'America/Pangnirtung': 'Pangnirtung EDT (UTC-04:00)',
        'America/Paramaribo': 'Paramaribo SRT (UTC-03:00)',
        'America/Phoenix': 'Phoenix MST (UTC-07:00)',
        'America/Port-au-Prince': 'Port-au-Prince EST (UTC-05:00)',
        'America/Port_of_Spain': 'Port of Spain AST (UTC-04:00)',
        'America/Porto_Velho': 'Porto Velho AMT (UTC-04:00)',
        'America/Puerto_Rico': 'Puerto Rico AST (UTC-04:00)',
        'America/Rainy_River': 'Rainy River CDT (UTC-05:00)',
        'America/Rankin_Inlet': 'Rankin Inlet CDT (UTC-05:00)',
        'America/Recife': 'Recife BRT (UTC-03:00)',
        'America/Regina': 'Regina CST (UTC-06:00)',
        'America/Resolute': 'Resolute CDT (UTC-05:00)',
        'America/Rio_Branco': 'Rio Branco ACT (UTC-05:00)',
        'America/Santarem': 'Santarem BRT (UTC-03:00)',
        'America/Santiago': 'Santiago CLT (UTC-04:00)',
        'America/Santo_Domingo': 'Santo Domingo AST (UTC-04:00)',
        'America/Sao_Paulo': 'Sao Paulo BRT (UTC-03:00)',
        'America/Scoresbysund': 'Scoresbysund EGST (UTC+00:00)',
        'America/Sitka': 'Sitka AKDT (UTC-08:00)',
        'America/St_Barthelemy': 'St Barthelemy AST (UTC-04:00)',
        'America/St_Johns': 'St Johns NDT (UTC-02:30)',
        'America/St_Kitts': 'St Kitts AST (UTC-04:00)',
        'America/St_Lucia': 'St Lucia AST (UTC-04:00)',
        'America/St_Thomas': 'St Thomas AST (UTC-04:00)',
        'America/St_Vincent': 'St Vincent AST (UTC-04:00)',
        'America/Swift_Current': 'Swift Current CST (UTC-06:00)',
        'America/Tegucigalpa': 'Tegucigalpa CST (UTC-06:00)',
        'America/Thule': 'Thule ADT (UTC-03:00)',
        'America/Thunder_Bay': 'Thunder Bay EDT (UTC-04:00)',
        'America/Tijuana': 'Tijuana PDT (UTC-07:00)',
        'America/Toronto': 'Toronto EDT (UTC-04:00)',
        'America/Tortola': 'Tortola AST (UTC-04:00)',
        'America/Vancouver': 'Vancouver PDT (UTC-07:00)',
        'America/Whitehorse': 'Whitehorse PDT (UTC-07:00)',
        'America/Winnipeg': 'Winnipeg CDT (UTC-05:00)',
        'America/Yakutat': 'Yakutat AKDT (UTC-08:00)',
        'America/Yellowknife': 'Yellowknife MDT (UTC-06:00)',
      },
      'Antarctica': {
        'Antarctica/Casey': 'Casey AWST (UTC+08:00)',
        'Antarctica/Davis': 'Davis DAVT (UTC+07:00)',
        'Antarctica/DumontDUrville': 'DumontDUrville DDUT (UTC+10:00)',
        'Antarctica/Macquarie': 'Macquarie MIST (UTC+11:00)',
        'Antarctica/Mawson': 'Mawson MAWT (UTC+05:00)',
        'Antarctica/McMurdo': 'McMurdo NZST (UTC+12:00)',
        'Antarctica/Palmer': 'Palmer CLT (UTC-04:00)',
        'Antarctica/Rothera': 'Rothera ROTT (UTC-03:00)',
        'Antarctica/Syowa': 'Syowa SYOT (UTC+03:00)',
        'Antarctica/Troll': 'Troll CEST (UTC+02:00)',
        'Antarctica/Vostok': 'Vostok VOST (UTC+06:00)'
      },
      'Arctic': {
        'Arctic/Longyearbyen': 'Longyearbyen CEST (UTC+02:00)'
      },
      'Asia': {
        'Asia/Aden': 'Aden AST (UTC+03:00)',
        'Asia/Almaty': 'Almaty +06 (UTC+06:00)',
        'Asia/Amman': 'Amman EEST (UTC+03:00)',
        'Asia/Anadyr': 'Anadyr ANAT (UTC+12:00)',
        'Asia/Aqtau': 'Aqtau +05 (UTC+05:00)',
        'Asia/Aqtobe': 'Aqtobe +05 (UTC+05:00)',
        'Asia/Ashgabat': 'Ashgabat TMT (UTC+05:00)',
        'Asia/Baghdad': 'Baghdad AST (UTC+03:00)',
        'Asia/Bahrain': 'Bahrain AST (UTC+03:00)',
        'Asia/Baku': 'Baku AZT (UTC+04:00)',
        'Asia/Bangkok': 'Bangkok ICT (UTC+07:00)',
        'Asia/Barnaul': 'Barnaul +07 (UTC+07:00)',
        'Asia/Beirut': 'Beirut EEST (UTC+03:00)',
        'Asia/Bishkek': 'Bishkek KGT (UTC+06:00)',
        'Asia/Brunei': 'Brunei BNT (UTC+08:00)',
        'Asia/Chita': 'Chita YAKT (UTC+09:00)',
        'Asia/Choibalsan': 'Choibalsan CHOST (UTC+09:00)',
        'Asia/Colombo': 'Colombo IST (UTC+05:30)',
        'Asia/Damascus': 'Damascus EEST (UTC+03:00)',
        'Asia/Dhaka': 'Dhaka BDT (UTC+06:00)',
        'Asia/Dili': 'Dili TLT (UTC+09:00)',
        'Asia/Dubai': 'Dubai GST (UTC+04:00)',
        'Asia/Dushanbe': 'Dushanbe TJT (UTC+05:00)',
        'Asia/Gaza': 'Gaza EEST (UTC+03:00)',
        'Asia/Hebron': 'Hebron EEST (UTC+03:00)',
        'Asia/Ho_Chi_Minh': 'Ho Chi Minh ICT (UTC+07:00)',
        'Asia/Hong_Kong': 'Hong Kong HKT (UTC+08:00)',
        'Asia/Hovd': 'Hovd HOVST (UTC+08:00)',
        'Asia/Irkutsk': 'Irkutsk IRKT (UTC+08:00)',
        'Asia/Jakarta': 'Jakarta WIB (UTC+07:00)',
        'Asia/Jayapura': 'Jayapura WIT (UTC+09:00)',
        'Asia/Jerusalem': 'Jerusalem IDT (UTC+03:00)',
        'Asia/Kabul': 'Kabul AFT (UTC+04:30)',
        'Asia/Kamchatka': 'Kamchatka PETT (UTC+12:00)',
        'Asia/Karachi': 'Karachi PKT (UTC+05:00)',
        'Asia/Kathmandu': 'Kathmandu NPT (UTC+05:45)',
        'Asia/Khandyga': 'Khandyga YAKT (UTC+09:00)',
        'Asia/Kolkata': 'Kolkata IST (UTC+05:30)',
        'Asia/Krasnoyarsk': 'Krasnoyarsk KRAT (UTC+07:00)',
        'Asia/Kuala_Lumpur': 'Kuala Lumpur MYT (UTC+08:00)',
        'Asia/Kuching': 'Kuching MYT (UTC+08:00)',
        'Asia/Kuwait': 'Kuwait AST (UTC+03:00)',
        'Asia/Macau': 'Macau CST (UTC+08:00)',
        'Asia/Magadan': 'Magadan MAGT (UTC+11:00)',
        'Asia/Makassar': 'Makassar WITA (UTC+08:00)',
        'Asia/Manila': 'Manila PHT (UTC+08:00)',
        'Asia/Muscat': 'Muscat GST (UTC+04:00)',
        'Asia/Nicosia': 'Nicosia EEST (UTC+03:00)',
        'Asia/Novokuznetsk': 'Novokuznetsk KRAT (UTC+07:00)',
        'Asia/Novosibirsk': 'Novosibirsk NOVT (UTC+06:00)',
        'Asia/Omsk': 'Omsk OMST (UTC+06:00)',
        'Asia/Oral': 'Oral +05 (UTC+05:00)',
        'Asia/Phnom_Penh': 'Phnom Penh ICT (UTC+07:00)',
        'Asia/Pontianak': 'Pontianak WIB (UTC+07:00)',
        'Asia/Pyongyang': 'Pyongyang KST (UTC+08:30)',
        'Asia/Qatar': 'Qatar AST (UTC+03:00)',
        'Asia/Qyzylorda': 'Qyzylorda +06 (UTC+06:00)',
        'Asia/Rangoon': 'Rangoon MMT (UTC+06:30)',
        'Asia/Riyadh': 'Riyadh AST (UTC+03:00)',
        'Asia/Sakhalin': 'Sakhalin SAKT (UTC+11:00)',
        'Asia/Samarkand': 'Samarkand UZT (UTC+05:00)',
        'Asia/Seoul': 'Seoul KST (UTC+09:00)',
        'Asia/Shanghai': 'Shanghai CST (UTC+08:00)',
        'Asia/Singapore': 'Singapore SGT (UTC+08:00)',
        'Asia/Srednekolymsk': 'Srednekolymsk SRET (UTC+11:00)',
        'Asia/Taipei': 'Taipei CST (UTC+08:00)',
        'Asia/Tashkent': 'Tashkent UZT (UTC+05:00)',
        'Asia/Tbilisi': 'Tbilisi GET (UTC+04:00)',
        'Asia/Tehran': 'Tehran IRDT (UTC+04:30)',
        'Asia/Thimphu': 'Thimphu BTT (UTC+06:00)',
        'Asia/Tokyo': 'Tokyo JST (UTC+09:00)',
        'Asia/Tomsk': 'Tomsk +07 (UTC+07:00)',
        'Asia/Ulaanbaatar': 'Ulaanbaatar ULAST (UTC+09:00)',
        'Asia/Urumqi': 'Urumqi XJT (UTC+06:00)',
        'Asia/Ust-Nera': 'Ust-Nera VLAT (UTC+10:00)',
        'Asia/Vientiane': 'Vientiane ICT (UTC+07:00)',
        'Asia/Vladivostok': 'Vladivostok VLAT (UTC+10:00)',
        'Asia/Yakutsk': 'Yakutsk YAKT (UTC+09:00)',
        'Asia/Yekaterinburg': 'Yekaterinburg YEKT (UTC+05:00)',
        'Asia/Yerevan': 'Yerevan AMT (UTC+04:00)'
      },
      'Atlantic': {
        'Atlantic/Azores': 'Azores AZOST (UTC+00:00)',
        'Atlantic/Bermuda': 'Bermuda ADT (UTC-03:00)',
        'Atlantic/Canary': 'Canary WEST (UTC+01:00)',
        'Atlantic/Cape_Verde': 'Cape Verde CVT (UTC-01:00)',
        'Atlantic/Faroe': 'Faroe WEST (UTC+01:00)',
        'Atlantic/Madeira': 'Madeira WEST (UTC+01:00)',
        'Atlantic/Reykjavik': 'Reykjavik GMT (UTC+00:00)',
        'Atlantic/South_Georgia': 'South Georgia GST (UTC-02:00)',
        'Atlantic/St_Helena': 'St Helena GMT (UTC+00:00)',
        'Atlantic/Stanley': 'Stanley FKST (UTC-03:00)'
      },
      'Australia': {
        'Australia/Adelaide': 'Adelaide ACST (UTC+09:30)',
        'Australia/Brisbane': 'Brisbane AEST (UTC+10:00)',
        'Australia/Broken_Hill': 'Broken Hill ACST (UTC+09:30)',
        'Australia/Currie': 'Currie AEST (UTC+10:00)',
        'Australia/Darwin': 'Darwin ACST (UTC+09:30)',
        'Australia/Eucla': 'Eucla ACWST (UTC+08:45)',
        'Australia/Hobart': 'Hobart AEST (UTC+10:00)',
        'Australia/Lindeman': 'Lindeman AEST (UTC+10:00)',
        'Australia/Lord_Howe': 'Lord Howe LHST (UTC+10:30)',
        'Australia/Melbourne': 'Melbourne AEST (UTC+10:00)',
        'Australia/Perth': 'Perth AWST (UTC+08:00)',
        'Australia/Sydney': 'Sydney AEST (UTC+10:00)'
      },
      'Europe': {
        'Europe/Amsterdam': 'Amsterdam CEST (UTC+02:00)',
        'Europe/Andorra': 'Andorra CEST (UTC+02:00)',
        'Europe/Astrakhan': 'Astrakhan +04 (UTC+04:00)',
        'Europe/Athens': 'Athens EEST (UTC+03:00)',
        'Europe/Belgrade': 'Belgrade CEST (UTC+02:00)',
        'Europe/Berlin': 'Berlin CEST (UTC+02:00)',
        'Europe/Bratislava': 'Bratislava CEST (UTC+02:00)',
        'Europe/Brussels': 'Brussels CEST (UTC+02:00)',
        'Europe/Bucharest': 'Bucharest EEST (UTC+03:00)',
        'Europe/Budapest': 'Budapest CEST (UTC+02:00)',
        'Europe/Busingen': 'Busingen CEST (UTC+02:00)',
        'Europe/Chisinau': 'Chisinau EEST (UTC+03:00)',
        'Europe/Copenhagen': 'Copenhagen CEST (UTC+02:00)',
        'Europe/Dublin': 'Dublin IST (UTC+01:00)',
        'Europe/Gibraltar': 'Gibraltar CEST (UTC+02:00)',
        'Europe/Guernsey': 'Guernsey BST (UTC+01:00)',
        'Europe/Helsinki': 'Helsinki EEST (UTC+03:00)',
        'Europe/Isle_of_Man': 'Isle of Man BST (UTC+01:00)',
        'Europe/Istanbul': 'Istanbul EEST (UTC+03:00)',
        'Europe/Jersey': 'Jersey BST (UTC+01:00)',
        'Europe/Kaliningrad': 'Kaliningrad EET (UTC+02:00)',
        'Europe/Kiev': 'Kiev EEST (UTC+03:00)',
        'Europe/Kirov': 'Kirov +03 (UTC+03:00)',
        'Europe/Lisbon': 'Lisbon WEST (UTC+01:00)',
        'Europe/Ljubljana': 'Ljubljana CEST (UTC+02:00)',
        'Europe/London': 'London BST (UTC+01:00)',
        'Europe/Luxembourg': 'Luxembourg CEST (UTC+02:00)',
        'Europe/Madrid': 'Madrid CEST (UTC+02:00)',
        'Europe/Malta': 'Malta CEST (UTC+02:00)',
        'Europe/Mariehamn': 'Mariehamn EEST (UTC+03:00)',
        'Europe/Minsk': 'Minsk MSK (UTC+03:00)',
        'Europe/Monaco': 'Monaco CEST (UTC+02:00)',
        'Europe/Moscow': 'Moscow MSK (UTC+03:00)',
        'Europe/Oslo': 'Oslo CEST (UTC+02:00)',
        'Europe/Paris': 'Paris CEST (UTC+02:00)',
        'Europe/Podgorica': 'Podgorica CEST (UTC+02:00)',
        'Europe/Prague': 'Prague CEST (UTC+02:00)',
        'Europe/Riga': 'Riga EEST (UTC+03:00)',
        'Europe/Rome': 'Rome CEST (UTC+02:00)',
        'Europe/Samara': 'Samara SAMT (UTC+04:00)',
        'Europe/San_Marino': 'San Marino CEST (UTC+02:00)',
        'Europe/Sarajevo': 'Sarajevo CEST (UTC+02:00)',
        'Europe/Simferopol': 'Simferopol MSK (UTC+03:00)',
        'Europe/Skopje': 'Skopje CEST (UTC+02:00)',
        'Europe/Sofia': 'Sofia EEST (UTC+03:00)',
        'Europe/Stockholm': 'Stockholm CEST (UTC+02:00)',
        'Europe/Tallinn': 'Tallinn EEST (UTC+03:00)',
        'Europe/Tirane': 'Tirane CEST (UTC+02:00)',
        'Europe/Ulyanovsk': 'Ulyanovsk +04 (UTC+04:00)',
        'Europe/Uzhgorod': 'Uzhgorod EEST (UTC+03:00)',
        'Europe/Vaduz': 'Vaduz CEST (UTC+02:00)',
        'Europe/Vatican': 'Vatican CEST (UTC+02:00)',
        'Europe/Vienna': 'Vienna CEST (UTC+02:00)',
        'Europe/Vilnius': 'Vilnius EEST (UTC+03:00)',
        'Europe/Volgograd': 'Volgograd MSK (UTC+03:00)',
        'Europe/Warsaw': 'Warsaw CEST (UTC+02:00)',
        'Europe/Zagreb': 'Zagreb CEST (UTC+02:00)',
        'Europe/Zaporozhye': 'Zaporozhye EEST (UTC+03:00)',
        'Europe/Zurich': 'Zurich CEST (UTC+02:00)',
      },
      'Indian': {
        'Indian/Antananarivo': 'Antananarivo EAT (UTC+03:00)',
        'Indian/Chagos': 'Chagos IOT (UTC+06:00)',
        'Indian/Christmas': 'Christmas CXT (UTC+07:00)',
        'Indian/Cocos': 'Cocos CCT (UTC+06:30)',
        'Indian/Comoro': 'Comoro EAT (UTC+03:00)',
        'Indian/Kerguelen': 'Kerguelen TFT (UTC+05:00)',
        'Indian/Mahe': 'Mahe SCT (UTC+04:00)',
        'Indian/Maldives': 'Maldives MVT (UTC+05:00)',
        'Indian/Mauritius': 'Mauritius MUT (UTC+04:00)',
        'Indian/Mayotte': 'Mayotte EAT (UTC+03:00)',
        'Indian/Reunion': 'Reunion RET (UTC+04:00)'
      },
      'Pacific': {
        'Pacific/Apia': 'Apia WSST (UTC+13:00)',
        'Pacific/Auckland': 'Auckland NZST (UTC+12:00)',
        'Pacific/Bougainville': 'Bougainville BST (UTC+11:00)',
        'Pacific/Chatham': 'Chatham CHAST (UTC+12:45)',
        'Pacific/Chuuk': 'Chuuk CHUT (UTC+10:00)',
        'Pacific/Easter': 'Easter EAST (UTC-06:00)',
        'Pacific/Efate': 'Efate VUT (UTC+11:00)',
        'Pacific/Enderbury': 'Enderbury PHOT (UTC+13:00)',
        'Pacific/Fakaofo': 'Fakaofo TKT (UTC+13:00)',
        'Pacific/Fiji': 'Fiji FJT (UTC+12:00)',
        'Pacific/Funafuti': 'Funafuti TVT (UTC+12:00)',
        'Pacific/Galapagos': 'Galapagos GALT (UTC-06:00)',
        'Pacific/Gambier': 'Gambier GAMT (UTC-09:00)',
        'Pacific/Guadalcanal': 'Guadalcanal SBT (UTC+11:00)',
        'Pacific/Guam': 'Guam ChST (UTC+10:00)',
        'Pacific/Honolulu': 'Honolulu HST (UTC-10:00)',
        'Pacific/Johnston': 'Johnston HST (UTC-10:00)',
        'Pacific/Kiritimati': 'Kiritimati LINT (UTC+14:00)',
        'Pacific/Kosrae': 'Kosrae KOST (UTC+11:00)',
        'Pacific/Kwajalein': 'Kwajalein MHT (UTC+12:00)',
        'Pacific/Majuro': 'Majuro MHT (UTC+12:00)',
        'Pacific/Marquesas': 'Marquesas MART (UTC-09:30)',
        'Pacific/Midway': 'Midway SST (UTC-11:00)',
        'Pacific/Nauru': 'Nauru NRT (UTC+12:00)',
        'Pacific/Niue': 'Niue NUT (UTC-11:00)',
        'Pacific/Norfolk': 'Norfolk NFT (UTC+11:00)',
        'Pacific/Noumea': 'Noumea NCT (UTC+11:00)',
        'Pacific/Pago_Pago': 'Pago Pago SST (UTC-11:00)',
        'Pacific/Palau': 'Palau PWT (UTC+09:00)',
        'Pacific/Pitcairn': 'Pitcairn PST (UTC-08:00)',
        'Pacific/Pohnpei': 'Pohnpei PONT (UTC+11:00)',
        'Pacific/Port_Moresby': 'Port Moresby PGT (UTC+10:00)',
        'Pacific/Rarotonga': 'Rarotonga CKT (UTC-10:00)',
        'Pacific/Saipan': 'Saipan ChST (UTC+10:00)',
        'Pacific/Tahiti': 'Tahiti TAHT (UTC-10:00)',
        'Pacific/Tarawa': 'Tarawa GILT (UTC+12:00)',
        'Pacific/Tongatapu': 'Tongatapu TOT (UTC+13:00)',
        'Pacific/Wake': 'Wake WAKT (UTC+12:00)',
        'Pacific/Wallis': 'Wallis WFT (UTC+12:00)'
      },
      'Other': {
        'UTC': 'UTC'
      }
    };
  }

  /**
   * {@inheritdoc}
   */
  formatChoices(
    timezones: { [key: string]: { [key: string] : string } }
  ): { [key: string]: { [key: string] : string } } {
    return timezones;
  }
}

export = TimezoneField
